#!/bin/bash
#
# Shell Script zur Ansteurung von 16*2 HD44780 LCD über PCF8574
#
# Das ganze ist mehr ein proof of concept
#
# Reiner Wehner 01.12.2013 V0.01
#
#!/bin/bash

#Syntax Ausgabe
usage() { echo -e "Usage: $0 [-a <Chipadr.Def.0x27] 
                   [-b <I2CBus Def.1>] 
                   [-s <string>] 
                   [-r <row 0|1|2 Def.1 0=cur.Pos>] 
                   [-i(nit)] 
                   [-l <BackLight 0|1> Def.1]
                   [-d debug, only echo] " 1>&2; exit 1; }


#defaults parameters
a=0x27     # Chip ADR
b=1	       # I2C Bus
r=0        # row 0=cur.pos 1=row1 pos1 2=row2 pos1
i=false    # init
l=1        # BackLight 0=off 1=on         
d=false    # no debug

#LCD WIRE
LCD_EN=0x10		# bit 4
LCD_RS=0x40		# bit 6

# LCD Commands
LCD_CLEARDISPLAY=0x01
LCD_RETURNHOME=0x02
LCD_ENTRYMODESET=0x04
LCD_DISPLAYCONTROL=0x08
LCD_CURSORSHIFT=0x10
LCD_FUNCTIONSET=0x20
LCD_SETCGRAMADDR=0x40
LCD_SETDDRAMADDR=0x80

# Flags for display on/off control
LCD_DISPLAYON=0x04
LCD_DISPLAYOFF=0x00
LCD_CURSORON=0x02
LCD_CURSOROFF=0x00
LCD_BLINKON=0x01
LCD_BLINKOFF=0x00

# Flags for display entry mode
LCD_ENTRYRIGHT=0x00
LCD_ENTRYLEFT=0x02
LCD_ENTRYSHIFTINCREMENT=0x01
LCD_ENTRYSHIFTDECREMENT=0x00

# Flags for display/cursor shift
LCD_DISPLAYMOVE=0x08
LCD_CURSORMOVE=0x00
LCD_MOVERIGHT=0x04
LCD_MOVELEFT=0x00
    
# flags for function set
LCD_8BITMODE=0x10
LCD_4BITMODE=0x00
LCD_2LINE=0x08
LCD_1LINE=0x00
LCD_5x10DOTS=0x04
LCD_5x8DOTS=0x00

#LCD variables
LCD_string=
LCD_adrrow2home=0x40
LCD_row2home=$(($LCD_SETDDRAMADDR | $LCD_adrrow2home))  #HD44780 S.21 Fig.8 


# flags for backlight control
LCD_BACKLIGHT=0x00    #bit 7 inverted by mjkdz board
LCD_NOBACKLIGHT=0x80  #bit 7 inverted by mjkdz board
LCD_BL=$LCD_BACKLIGHT  #Default on                              

# other variables
i2cDataBlock=         #DataBlock for i2cset


#generate i2cDataBlock with clock 
function getLcdByteClock_byte() {
  putByte=$1 
	i2cDataBlock+=" $(($putByte))"
	i2cDataBlock+=" $(($putByte | LCD_EN))"
	i2cDataBlock+=" $(($putByte))"
}

#write data byte to lcd RS=1
function writeData_byte() {
   SendByte=$1
   HByte=$(($SendByte >> 4))
   LByte=$(($SendByte & 0xF))
    
   #first Nibble and BL and RS
   getLcdByteClock_byte $(($HByte | $LCD_BL | $LCD_RS))
   #second Nibble and BL and RS
   getLcdByteClock_byte $(($LByte | $LCD_BL | $LCD_RS))  
   
   writeOutBlock_i2cDataBlock  #write out data       
}

#write command byte to lcd RS=0
function writeCmd_byte() {
   SendByte=$1
   HByte=$(($SendByte >> 4))
   LByte=$(($SendByte & 0xF))
   
   getLcdByteClock_byte $(($HByte | $LCD_BL))
   getLcdByteClock_byte $(($LByte | $LCD_BL))  

   writeOutBlock_i2cDataBlock  #write out data    
}

#write out data, i2cDataBlock always in i2cDataBlock
function  writeOutBlock_i2cDataBlock {
   if [ $d == true ]  
      then
         echo "i2cset -y ${b} $((${a})) ${i2cDataBlock} i" #debug
      else
         i2cset -y ${b} $((${a})) ${i2cDataBlock} i
   fi

   i2cDataBlock=   #clear i2cDataBlock
}   

function initLcd() {
   #Bus init HD44780 S.46 Fig. 24 4-Bit-Interface 
   getLcdByteClock_byte 0x03
   writeOutBlock_i2cDataBlock 
   
   getLcdByteClock_byte 0x03
   writeOutBlock_i2cDataBlock
   
   getLcdByteClock_byte 0x03
   writeOutBlock_i2cDataBlock
   
   getLcdByteClock_byte 0x02
   writeOutBlock_i2cDataBlock
   
   #Bus init HD44780 S.46 Fig. 24 4-Bit-Interface Function set (last 4 Bytes)
   writeCmd_byte $(($LCD_FUNCTIONSET | $LCD_4BITMODE  | $LCD_2LINE | $LCD_5x8DOTS))            #0x28 Set 4 bit, 2 line mode (Multi-line
   writeCmd_byte $(($LCD_DISPLAYCONTROL | $LCD_DISPLAYOFF | $LCD_CURSOROFF | $LCD_BLINKOFF))   #0x08 Hide cursor, don't blink
   writeCmd_byte ${LCD_CLEARDISPLAY}                                                           #0x01 Clear display, move cursor home
   writeCmd_byte $(($LCD_ENTRYMODESET | $LCD_ENTRYLEFT | $LCD_ENTRYSHIFTDECREMENT))            #0x06 Move cursor right	 
   writeCmd_byte $(($LCD_DISPLAYCONTROL | $LCD_DISPLAYON | $LCD_CURSOROFF | $LCD_BLINKOFF))    #0x0C Turn on display
}

if [ $# -eq 0 ]; then
   usage
fi

while getopts ":a:b:s:r:l:id" o; do
    case "${o}" in
        a)
            a=${OPTARG}
            ;;
        b)
            b=${OPTARG}
            ;;
        s)
            s=${OPTARG}
            LCD_string=$s
            ;;
        r)
            r=${OPTARG}
            ((r == 0 || r == 1 || r == 2)) || usage
            ;;            
        l)
            l=${OPTARG}
            ((l == 0 || l == 1)) || usage
            ;;
        i)
            i=true
            ;;           
        d)
            d=true
            ;;           
       \?) usage
            ;;
        *)
            usage
            ;;
    esac
done
shift $((OPTIND-1))

#LCD-BackLight
if [ $l -eq 0 ]; then
   LCD_BL=LCD_NOBACKLIGHT
fi

#LCD init if needed
if [ $i == true ]; then
   initLcd
fi

#cursor setzen 
case "${r}" in
		1)
		   #clear display cursor home
		   writeCmd_byte ${LCD_CLEARDISPLAY} #0x01
		   ;;
		2)
			 #cursor row2pos1
		   writeCmd_byte ${LCD_row2home} #0xC0		 
       ;;
esac

#max 16 char from LCD_string to LCD
if [ -n "$LCD_string" ]; then
   i=0
   maxlenght=${#LCD_string}
   if [[ "$maxlenght" -gt 16 ]]; then
     maxlenght=16
   fi
   actstring=
   while [ $i -lt $maxlenght ]
   do
      actstring=${LCD_string:$i:1}

      #put char
      writeData_byte $(printf "%d" "'${actstring}")

      ((i++))
   done
fi
exit 0